//*******************************************************************************
//* Dieses Programm wurde von Ulrich Radig fr Atmel AVR C geschrieben 	 	*
//* 																			*
//* Ich wnsche jeden viel Spa mit diesen kleinen Web-Server der ohne 			*
//* zustzlichen Speicher auskommt.												*
//*																				*
//* Bei Fragen und Verbesserungen wendet euch per EMail an						*
//*																				*
//* mail@ulrichradig.de															*
//*																				*
//* oder im Forum meiner Web Page : www.ulrichradig.de							*
//*																				*
//*******************************************************************************

//Ethernet Card Ports
#define data_ctlr_rtl  			DDRA
#define data_addr_rtl			DDRC	//Fr ATMega128 wird bei ATMega102 nicht bentigt
#define addr_port_rtl  			PORTC
#define data_port_rtl_write  	PORTA
#define data_port_rtl_read 		PINA	//Read
#define write_pin				6		//gibt den Pin an wo der WriteIO Signal angeschlossen ist
#define read_pin				5		//gibt den Pin an wo der ReadIO Signal angeschlossen ist
#define reset_pin				7		//gibt den Pin an wo der Reset angeschlossen ist
#define input					0x00	//setzt einen Port auf Input
#define output					0xff	//setzt einen Port auf Output

//Ethernet Header Layout
#define	enetpacketdest0			0x00	//MAC Zieladresse 6 Byte (MSB)  
#define	enetpacketdest1			0x01	//MAC Zieladresse 
#define	enetpacketdest2			0x02	//MAC Zieladresse 
#define	enetpacketdest3			0x03	//MAC Zieladresse 
#define	enetpacketdest4			0x04	//MAC Zieladresse 
#define	enetpacketdest5			0x05	//MAC Zieladresse (LSB)
#define	enetpacketsrc0			0x06	//MAC Quelladresse 6 Byte (MSB)  
#define	enetpacketsrc1			0x07	//MAC Quelladresse
#define	enetpacketsrc2			0x08	//MAC Quelladresse
#define	enetpacketsrc3			0x09	//MAC Quelladresse
#define	enetpacketsrc4			0x0A	//MAC Quelladresse
#define	enetpacketsrc5			0x0B	//MAC Quelladresse (LSB)
#define	enetpacketType0			0x0C	//2 Byte Packettyp
#define	enetpacketType1			0x0D	//Nutzlast 0x0800=IP Datagramm;0x0806 = ARP 
#define enetpacketData			0x0E	//Anfang von den Nutzlast Data

//ARP Layout
#define	arp_hwtype			  	0x0E	//Hardware Typ enthlt den Code fr Ethernet oder andere Link Layer
#define	arp_prtype				0x10	//Protokoll Typ enthlt den Code fr IP o. anderes bertragungsprotokoll
#define	arp_hwlen			  	0x12	//Lnge der Hardwareadresse enthlt 6 fr 6 Byte lange MAC Adressen
#define	arp_prlen			  	0x13	//Lnge der Protocolladresse enthlt 4 fr 4 Byte lange IP Adressen
#define	arp_op				  	0x14	//Enthlt Code der signalisiert ob es sich um eine Anfrage o. Antwort handelt
#define	arp_shaddr			  	0x16	//Enthlt die MAC Adresse des Anfragenden   
#define	arp_sipaddr			  	0x1C	//Enthlt die IP Adresse des Absenders
#define	arp_thaddr			  	0x20	//MAC Adresse Ziel, ist in diesem Fall 6 * 00,da die Adresse erst noch herausgefunden werden soll (ARP Request)  
#define	arp_tipaddr				0x26	//IP Adresse enthlt die IP Adresse zu der die Kommunikation aufgebaut werden soll 

//IP Header Layout
#define	ip_vers_len				0x0E	//4 BIT Die Versionsnummer von IP, meistens also 4 + 4Bit Headergre 	
#define	ip_tos					0x0F	//Type of Service
#define	ip_pktlen				0x10	//16 Bit Komplette Lng des IP Datagrams in Bytes
#define	ip_id					0x12	//ID des Packet fr Fragmentierung oder Reassemblierung	
#define	ip_frag_offset			0x14	//wird benutzt um ein fragmentiertes IP Packet wieder korrekt zusammenzusetzen	
#define	ip_ttl					0x16	//8 Bit Time to Live die lebenszeit eines Paket
#define	ip_proto				0x17	//Zeigt das hherschichtige Protokoll an (TCP, UDP, ICMP)	
#define	ip_hdr_cksum			0x18	//Checksumme des IP Headers	
#define	ip_srcaddr				0x1A	//32 Bit IP Quelladresse
#define	ip_destaddr				0x1E	//32 Bit IP Zieladresse
#define	ip_data					0x22	//beginn der IP Daten (hier fngt dann UDP TCP oder ICMP an)	

//IP Protocol Types
#define	PROT_ICMP				0x01	//zeigt an die Nutzlasten enthalten das ICMP Protokoll
#define	PROT_TCP				0x06	//zeigt an die Nutzlasten enthalten das TCP Protokoll
#define	PROT_UDP				0x11	//zeigt an die Nutzlasten enthalten das UDP Protokoll

//ICMP Header
#define	ICMP_type				0x22	//8 bit typ identifiziert Aufgabe der Nachricht 0 = Echo Reply 8 = Echo Request
#define	ICMP_code				0x23	//8 Bit Code enthlt Detailangaben zu bestimmten Typen
#define	ICMP_cksum				0x24	//16 Bit Prfsumme enthlt die CRC Prfsumme
#define	ICMP_id					0x26	//1 byte Identifizierung
#define	ICMP_seqnum				0x28	//Sequenznummer
#define ICMP_data				0x2A	//ab hier knnen sich Testdaten befinden (Echo/Request)

//TCP Header Layout
#define	TCP_srcport				0x22	//der Quellport fr die Verbindung (Socket)
#define	TCP_destport			0x24	//der Zielport fr die Verbindung (Socket)
#define	TCP_seqnum				0x26	//numerierter Bytestrom
#define	TCP_acknum				0x2A	//numerierter Bytestrom (Gegenrichtung)
#define	TCP_hdrflags			0x2E	//4Bit Lnge des Headers in 32 Bit schritten rest Flags
#define	TCP_window				0x30	//Fenstergre max. 64K Byte
#define	tcp_chksum				0x32	//Enthlt eine Prfsumme ber Header und Datenbereich
#define	TCP_urgentptr			0x34	//16 Bit fr dringende bertragung
#define TCP_data				0x36	//Ab hier beginnt der Datenbereich

//TCP Flags
#define  FIN_Flag				0x01	//Datenbertragung beendet (finished)
#define  SYN_Flag				0x02	//Verbindungs aufbauen (synchronize)
#define  RST_Flag				0x04	//Verbindung neu initialisieren (reset)
#define  PSH_Flag				0x08	//Datenbergabe an Anwendung beschleunigen (push)
#define  ACK_Flag				0x10	//Datenbertragung besttigen (acknowledged)

//Realtek Netzwerkkarten Register
#define cr						0x00
#define pstart					0x01
#define par0					0x01                                          
#define pstop					0x02
#define bnry					0x03
#define tpsr					0x04
#define tbcr0					0x05
#define tbcr1					0x06
#define isr						0x07
#define curr					0x07                                       
#define rsar0					0x08
#define crda0					0x08
#define rsar1					0x09	
#define crdal					0x09
#define rbcr0					0x0A
#define rbcr1					0x0B
#define rsr						0x0C
#define rcr						0x0C
#define tcr						0x0D
#define cntr0					0x0D
#define dcr						0x0E
#define cntr1					0x0E
#define imr						0x0F
#define cntr2					0x0F
#define rdmaport				0x10
#define rstport					0x18
//RTL8019AS initial register Werte
#define rcrval					0x04
#define tcrval					0x00
#define dcrval					0x58
//PXR and OVW interrupt enabled
#define imrval					0x11
#define txstart					0x40
#define rxstart					0x46
#define rxstop					0x60
//RTL8019AS ISR Register Bits
#define rst						7
#define rdc						6
#define ovw						4
#define prx						0

